Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Security.Principal
Imports System.Web
Imports Microsoft.ReportingServices.Interfaces

Namespace MSSQLRS.FormsSecurity

    Public Class AuthenticationExtension : Implements IAuthenticationExtension
        ' The sample is based on Microsoft's Forms Authentication Sample for Reporting Services. 
        ' The original sample was written in C#. It has been translated it into Visual Basic 
        ' for consistency with the other examples in this book. Some revisions were also made 
        ' to better fit with the Galactic Delivery Services examples.

        Private Const ConnectionString As String = "Server=(local);UID=GalacticReporting;pwd=gds;database=Galactic"

        ' SetConfiguration is required by the implementation of IExtension
        Public Sub SetConfiguration(ByVal configuration As String) Implements IExtension.SetConfiguration

        End Sub

        ' LocalizedName is required by the implementation of IExtension
        Public ReadOnly Property LocalizedName() As String Implements IExtension.LocalizedName
            Get
                Return Nothing
            End Get
        End Property

        ' This function determines whether a user logon is valid.
        Public Function LogonUser(ByVal userName As String, ByVal password As String, ByVal authority As String) As Boolean Implements IAuthenticationExtension.LogonUser
            Return CheckAuthentication.VerifyPassword(userName, password)
        End Function

        ' GetUserInfo is required by the implementation of IAuthenticationExtension.
        ' The Report Server calls the GetUserInfo method for each request to 
        ' retrieve the current user identity.
        Public Sub GetUserInfo(ByRef userIdentity As IIdentity, ByRef userId As IntPtr) Implements IAuthenticationExtension.GetUserInfo
            ' If the current user identity is not null,
            ' set the userIdentity parameter to that of the current user.
            If (Not (HttpContext.Current Is Nothing)) And (Not (HttpContext.Current.User.Identity Is Nothing)) Then
                userIdentity = HttpContext.Current.User.Identity
            Else
                userIdentity = Nothing
            End If

            userId = IntPtr.Zero

        End Sub

        ' This function is called by the Report Server when it sets 
        ' security on an item. The function calls VerifyUser to make
        ' sure this is a valid user name.
        Public Function IsValidPrincipalName(ByVal principalName As String) As Boolean Implements IAuthenticationExtension.IsValidPrincipalName
            Return VerifyUser(principalName)
        End Function

        ' Look up the user name in the database to make sure it is valid.
        Public Shared Function VerifyUser(ByVal userName As String) As Boolean
            Dim isValid As Boolean = False
            Dim conn As SqlConnection = New SqlConnection(ConnectionString)
            Dim cmd As SqlCommand = New SqlCommand("stp_LookupUser", conn)
            Dim sqlParam As SqlParameter
            Dim reader As SqlDataReader

            ' Look up the user name in the Employee table 
            ' in the Galactic database.
            cmd.CommandType = CommandType.StoredProcedure

            sqlParam = cmd.Parameters.Add("@UserName", SqlDbType.VarChar, 255)
            sqlParam.Value = userName

            Try
                conn.Open()
                reader = cmd.ExecuteReader

                ' If a row was returned, the user is valid.
                If reader.Read() Then
                    isValid = True
                End If
            Catch ex As Exception
                Throw New Exception("Exception verifying password. " & ex.Message)
            Finally
                conn.Close()
            End Try

            Return isValid

        End Function

    End Class

End Namespace
