Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Security.Cryptography
Imports System.Web.Security

Namespace MSSQLRS.FormsSecurity
    Friend Class CheckAuthentication
        ' The sample is based on Microsoft's Forms Authentication Sample for Reporting Services. 
        ' The original sample was written in C#. It has been translated it into Visual Basic 
        ' for consistency with the other examples in this book. Some revisions were also made 
        ' to better fit with the Galactic Delivery Services examples.

        Private Const ConnectionString As String = "Server=(local);UID=GalacticReporting;pwd=gds;database=Galactic"

        ' This function is used to create a cryptographic random number
        ' that is used to salt the user's password for added security.
        Shared Function CreateSalt(ByVal size As Integer) As String
            ' Use the cryptographic service provider to generate 
            ' the cryptographic random number.
            Dim rng As RNGCryptoServiceProvider = New RNGCryptoServiceProvider
            Dim buff(size) As Byte

            rng.GetBytes(buff)

            ' Return a base64 representation of the random number.
            Return Convert.ToBase64String(buff)
        End Function

        ' This function returns a hash of the combined password and salt value
        Shared Function CreatePasswordHash(ByVal pwd As String, ByVal salt As String) As String
            Dim saltAndPwd As String
            Dim hashedPwd As String

            ' Concatenate the password and the salt value
            saltAndPwd = pwd & salt

            ' Hash the salted password
            hashedPwd = FormsAuthentication.HashPasswordForStoringInConfigFile(saltAndPwd, "SHA1")

            Return hashedPwd
        End Function

        ' This function validates a user name and password.
        Shared Function VerifyPassword(ByVal suppliedUserName As String, ByVal suppliedPassword As String) As Boolean
            Dim passwordMatch As Boolean = False
            Dim conn As SqlConnection = New SqlConnection(ConnectionString)
            Dim cmd As SqlCommand = New SqlCommand("stp_LookupUser", conn)
            Dim sqlParam As SqlParameter
            Dim reader As SqlDataReader
            Dim dbPasswordHash As String
            Dim dbPasswordSalt As String
            Dim passwordAndSalt As String
            Dim hashedPasswordAndSalt As String

            ' Get the PasswordHash and the PasswordSalt from 
            ' the Employee table in the Galactic database.
            cmd.CommandType = CommandType.StoredProcedure

            sqlParam = cmd.Parameters.Add("@UserName", SqlDbType.VarChar, 255)
            sqlParam.Value = suppliedUserName

            Try
                conn.Open()
                reader = cmd.ExecuteReader

                ' Advance the reader to the one and only row in the result set.
                If reader.Read() Then

                    ' Get the values from the result set.
                    dbPasswordHash = reader.GetString(0)
                    dbPasswordSalt = reader.GetString(1)

                    reader.Close()

                    ' Concatenate the password and the salt.
                    passwordAndSalt = suppliedPassword & dbPasswordSalt

                    ' Hash the concatenated string
                    hashedPasswordAndSalt = FormsAuthentication.HashPasswordForStoringInConfigFile(passwordAndSalt, "SHA1")

                    ' Compare the two hashed values.
                    If hashedPasswordAndSalt = dbPasswordHash Then
                        passwordMatch = True
                    Else
                        passwordMatch = False
                    End If
                Else
                    passwordMatch = False
                End If
            Catch ex As Exception
                Throw New Exception("Exception verifying password. " & ex.Message)
            Finally
                conn.Close()
            End Try

            Return passwordMatch

        End Function

        ' Make sure that there are no unwanted characters in the username or password.
        ' Also make sure that they are not too long.
        ' This prevents problems if MyReports is enabled and also can help prevent certain types 
        ' of security attachs.
        Public Shared Function ValidateString(ByVal stringToCheck As String) As Boolean

            If Len(stringToCheck) > 260 Then
                Return False
            ElseIf InStr(stringToCheck, ":") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "?") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, ";") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "@") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "&") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "=") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "+") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "$") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, ",") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "\") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "*") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, ">") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "<") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "|") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, ".") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, """") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "/") > 0 Then
                Return False
            ElseIf InStr(stringToCheck, "'") > 0 Then
                Return False
            End If

            Return True

        End Function

    End Class

End Namespace
